// SPDX-License-Identifier: GPL-2.0-or-later

#ifndef PROTOS_H
#define PROTOS_H

#include <elf.h>
#ifndef DT_MIPS_RLD_MAP_REL
#define DT_MIPS_RLD_MAP_REL 0x70000035
#endif
#include <stdbool.h>
#include "config.h"

#ifdef WORDS_BIGENDIAN
#define ELFDATA2 ELFDATA2MSB
#else
#define ELFDATA2 ELFDATA2LSB
#endif

typedef union {
  unsigned char e_ident[EI_NIDENT];
  Elf32_Ehdr e32;
  Elf64_Ehdr e64;
} ChrElf_Ehdr;

typedef union {
  Elf32_Shdr e32;
  Elf64_Shdr e64;
} ChrElf_Shdr;

typedef union {
  Elf32_Phdr e32;
  Elf64_Phdr e64;
} ChrElf_Phdr;

extern bool is_e32;
extern bool swap_bytes;

#define DO_SWAPU16(x) ( !swap_bytes ? (uint16_t)x : (uint16_t)__builtin_bswap16(x) )
#define DO_SWAPU32(x) ( !swap_bytes ? (uint32_t)x : (uint32_t)__builtin_bswap32(x) )
#define DO_SWAPU64(x) ( !swap_bytes ? (uint64_t)x : (uint64_t)__builtin_bswap64(x) )
#define DO_SWAPS16(x) ( !swap_bytes ? (int16_t)x : (int16_t)__builtin_bswap16(x) )
#define DO_SWAPS32(x) ( !swap_bytes ? (int32_t)x : (int32_t)__builtin_bswap32(x) )
#define DO_SWAPS64(x) ( !swap_bytes ? (int64_t)x : (int64_t)__builtin_bswap64(x) )

#define EHDRWS(x) (is_e32 ? DO_SWAPS32(ehdr.e32.x) : DO_SWAPS64(ehdr.e64.x))
#define EHDRHS(x) (is_e32 ? DO_SWAPS16(ehdr.e32.x) : DO_SWAPS16(ehdr.e64.x))
#define EHDRWU(x) (is_e32 ? DO_SWAPU32(ehdr.e32.x) : DO_SWAPU64(ehdr.e64.x))
#define EHDRHU(x) (is_e32 ? DO_SWAPU16(ehdr.e32.x) : DO_SWAPU16(ehdr.e64.x))
#define PHDR(x) (is_e32 ? DO_SWAPU32(phdr.e32.x) : DO_SWAPU64(phdr.e64.x))
#define SHDR_W(x) (is_e32 ? DO_SWAPU32(shdr.e32.x) : DO_SWAPU32(shdr.e64.x))
#define SHDR_O(x) (is_e32 ? DO_SWAPU32(shdr.e32.x) : DO_SWAPU64(shdr.e64.x))
#define DYNSU(i,x) (is_e32 ? DO_SWAPU32(((Elf32_Dyn *)dyns)[i].x) \
  : DO_SWAPU64(((Elf64_Dyn *)dyns)[i].x))
#define DYNSS(i,x) (is_e32 ? DO_SWAPS32(((Elf32_Dyn *)dyns)[i].x) \
  : DO_SWAPS64(((Elf64_Dyn *)dyns)[i].x))

int killrpath(const char *filename);
int chrpath(const char *filename, const char *newpath, bool convert);

int elf_open(const char *filename, int flags, ChrElf_Ehdr *ehdr);
void elf_close(int fd);
int elf_find_dynamic_section(int fd, ChrElf_Ehdr *ehdr, ChrElf_Phdr *phdr);
const char *elf_tagname(int tag);
int elf_dynpath_tag(int tag);

#endif /* PROTOS_H */
